#!/bin/sh

PATH=/sbin:/bin:/usr/bin:/usr/sbin
TMPDIR=/mnt/ramdisk
BACKUPFILE=${TMPDIR}/backuptrackerconfig
VERSIONFILE=../pkgVersion.xml
VERSION=`/usr/bin/tinyxmlparser -x /tracker/conf.d -f ${VERSIONFILE}`
MODEL=`confclient -g system_info_extendedmodelname -p 9 -t Value`
OPENSSLKEY="${MODEL}-${VERSION}"

CONFCLIENT=/usr/sbin/confclient
CDF_BACKUP_OPTION="system_mvaas&system_location&reportpush&server&media&vadp_event_triggerlist&event&di&do&tracker&camctrl_c0_cameraid&uart_i0_baudrate&uart_i1_baudrate"
CDF_BACKUP_OPTION_FILE=${TMPDIR}/backupCDFlist.xml

Create_Backup_File()
{
    cd ../
    local userConfPath=conf.d/usrdef
	local jsonConfPath=conf.d/vca
	local trackerVersionPath=pkgVersion.xml
	local trackerConfigPath=tracker_config.xml
	local sensorCalibrationPath=/etc/conf.d/sensorcalibration_c0_coef.xml
	local sensorCalibrationDefaultPath=/etc/conf.d/sensorcalibration_c0_coef_default.xml
	local configSystemInfoPath=/etc/conf.d/config_systeminfo.xml
	${CONFCLIENT} -g "${CDF_BACKUP_OPTION}" -t XML -p 7 > ${CDF_BACKUP_OPTION_FILE}
    
	[ ! -d "${userConfPath}" ] || [ ! -d "${jsonConfPath}" ] || ] [ ! -e "${trackerVersionPath}" ] || [ ! -e "${CDF_BACKUP_OPTION_FILE}" ] || [ ! -e "${trackerConfigPath}" ] && return 1

	local allFilePath="${userConfPath} ${jsonConfPath} ${trackerVersionPath} ${CDF_BACKUP_OPTION_FILE} ${trackerConfigPath}"

	if [ -f "${sensorCalibrationPath}" ]
	then
		allFilePath="${allFilePath} ${sensorCalibrationPath}"
	fi

	if [ -f "${sensorCalibrationDefaultPath}" ]
	then
		allFilePath="${allFilePath} ${sensorCalibrationDefaultPath}"
	fi
	if [ -f "${configSystemInfoPath}" ]
	then
		allFilePath="${allFilePath} ${configSystemInfoPath}"
	fi

	tar jcf - ${allFilePath} | openssl enc -aes-256-cbc -out ${BACKUPFILE} -e -salt -k ${OPENSSLKEY}
    strlog=`ls -l ${BACKUPFILE} | awk -F' ' '{print $5}'`
    return 0   
}

if [ "$1" = "backup" ]
then
	Create_Backup_File
	[ $? -eq 0 ] && {
		echo -ne "Content-type: text/plain\r\n"
		echo -ne "Content-Disposition: attachment; filename=backupTrackerConfig-v${VERSION}\r\n"
		echo -ne "Content-length:${strlog}\r\n\r\n"
		cat $BACKUPFILE>&1
		rm -rf $BACKUPFILE
		rm -rf $CDF_BACKUP_OPTION_FILE
	}
fi
