#!/bin/sh

# System settings
PATH=/bin:/sbin:/usr/bin:/usr/sbin
NET_STATUS=/var/run/netstatus
SMTPC="/usr/bin/smtpclient"
FTPMPUT_PROG="/usr/bin/ftpmput"
FTPS_PROG="/usr/sbin/ftps"
HTTP_CLIENT_PROG="/usr/sbin/httpclient"
CURL_PROG="/sbin/curl"
SEND_FILE="/var/run/test.txt"
SMBMOUNT="/usr/sbin/smbmount"
UMOUNT="/bin/umount"
CONF_PROG="/usr/sbin/confclient"
SMTPS_CONF="/etc/conf.d/smtps.conf"
STUNNEL_PROG="/usr/sbin/stunnel"
TEMP_FILE="/mnt/ramdisk/event_mail_tempfile"
pidfile="/var/run/smtps.pid"
pemfile="/etc/conf.d/privatekey.pem"
certfile="/etc/conf.d/publickey.cert"
CONFCLIENT="/usr/sbin/confclient"
TINYXMLPARSER="/usr/bin/tinyxmlparser"
HTTPS_CLIENT_CONF="../conf.d/usrdef/https_client.conf"
HTTPS_CLIENT_PID_FILE="/var/run/httpsclient.pid"
[ -e ${CURL_PROG} ] && cURL="true"

resHtmlHead()
{
    echo -ne "Content-type: text/html\r\n"
    echo -ne "\r\n"
    echo -ne "<html>\n
              <head>\n
              <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n
              <script type='text/javascript' src='/include/common.js'></script>\n
              </head>\n
              <body>\n"
}

resHtmlTail()
{
    echo "</body></html>"
}

transHtml()
{
    echo -e "<script type="text/javascript">document.write(translator('$1'))</script>"
}

resHtml()
{
    resHtmlHead

    case "${resMsg}" in
        "checkNetFail")
	        transHtml "please_check_internet_connection"
            ;;
        "sendEmailFail")
            transHtml "error_in_sending_email"
            ;;
        "sendEmailSuccess")
            transHtml "the_email_has_been_sent_successfully"
            ;;
        "errEmailAddr")
            transHtml "please_define_mail_server_location"
            ;;
        "sendFtpSuccess")
            transHtml "ftp_transmission_successfully"
            ;;
        "sendFtpFail")
            transHtml "ftp_transmission_failed"
            ;;
        "errFtpAddr")
            transHtml "please_define_ftp_address"
            ;;
        "sendHttpSuccess")
            transHtml "http_transmission_successfully"
            ;;
        "sendHttpFail")
            transHtml "http_transmission_failed"
            ;;
        "errHttpAddr")
            transHtml "please_define_a_valid_http_address"
            ;;
        "mountNasSuccess")
			transHtml "mount_successfully"
            ;;
        "nasDenied")
			transHtml "mount_success_but_transmit_denied"
            ;;
        "mountNasFail")
		    transHtml "mount_failed"
            ;;
        "storeSdCardSuccess")
			transHtml "test_sd_card_successfully"
            ;;
        "storeSdCardFail")
            transHtml "test_sd_failed"
	        transHtml "the_device_is_not_existed"
	        transHtml "check_sd_card"
            ;;
        "invalidPasswdRefer")
            echo "invalid password reference"
            ;;
        *)
            ;;
    esac

    resHtmlTail
}

resJson()
{
    echo -ne "Content-type: application/json\r\n"
    echo -ne "\r\n"

    case "${resMsg}" in
        "sendEmailSuccess" | "sendFtpSuccess" | "sendHttpSuccess" | "mountNasSuccess" | "storeSdCardSuccess")
            echo "{\"testConnection\":\"success\"}"
            ;;
        "invalidPasswdRefer")
            echo "{\"testConnection\":\"invalid password reference\"}"
            ;;
        *)
            echo "{\"testConnection\":\"fail\"}"
            ;;
    esac
}

response()
{
    if [ "${resType}" == "json" ]; then
        resJson
    else
        resHtml
    fi
}

checkNetStatus()
{
    if [ ! -f "$NET_STATUS" ]; then
        echo "false"
    else
        echo "true"
    fi
}

getEncParam()
{
    if [ "$REQUEST_METHOD" = "POST" ]; then
	    strparam=`cat $stdin | cut -c1-$CONTENT_LENGTH`
    else
        strparam=$QUERY_STRING
    fi
}

decParam()
{
    strparam=`except_decode.sh $strparam`
}

parseParam()
{
    type=$(echo "$strparam" | sed -n 's/^.*type=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    address=$(echo "$strparam" | sed -n 's/^.*address=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    username=$(echo "$strparam" | sed -n 's/^.*username=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    passwd=$(echo "$strparam" | sed -n 's/^.*passwd=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    port=$(echo "$strparam" | sed -n 's/^.*port=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    sslmode=$(echo "$strparam" | sed -n 's/^.*sslmode=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    passive=$(echo "$strparam" | sed -n 's/^.*passive=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    location=$(echo "$strparam" | sed -n 's/^.*location=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    senderemail=$(echo "$strparam" | sed -n 's/^.*senderemail=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    recipientemail=$(echo "$strparam" | sed -n 's/^.*recipientemail=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    workgroup=$(echo "$strparam" | sed -n 's/^.*workgroup=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    url=$(echo "$strparam" | sed -n 's/^.*url=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    resType=$(echo "$strparam" | sed -n 's/^.*response=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    refTarget=$(echo "$strparam" | sed -n 's/^.*reftarget=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
    refPasswdByIndex=$(echo "$strparam" | sed -n 's/^.*refpasswdbyindex=\([^&]*\).*$/\1/p' | sed "s/%20/ /g")
}

updateReportPushPasswd()
{
    [ -z "${refPasswdByIndex}" ] && return

    pkgPath="$(dirname $(pwd))"
    config="${pkgPath}/conf.d/vca/config_vca_reportpush.xml"
    svrType="$(${TINYXMLPARSER} -x /reportpush/i${refPasswdByIndex}/servertypeselector -f ${config})"
    svrAddr="$(${TINYXMLPARSER} -x /reportpush/i${refPasswdByIndex}/url -f ${config})"
    svrPort="$(${TINYXMLPARSER} -x /reportpush/i${refPasswdByIndex}/port -f ${config})"
    svrUser="$(${TINYXMLPARSER} -x /reportpush/i${refPasswdByIndex}/usr -f ${config})"

    if [ "${svrType}" != "${type}" -o "${svrAddr}" != "${address}" -o "${svrPort}" != "${port}" -o "${svrUser}" != "${username}" ]; then
        resMsg="invalidPasswdRefer"
        return
    fi

    passwd="$(${TINYXMLPARSER} -x /reportpush/i${refPasswdByIndex}/pwd -f ${config})"
}

updateValidationPasswd()
{
    [ -z "${refPasswdByIndex}" ] && return

    pkgPath="$(dirname $(pwd))"
    config="${pkgPath}/conf.d/vca/config_validation.xml"
    svrAddr="$(${TINYXMLPARSER} -x /Validation/i${refPasswdByIndex}/FTP/Address -f ${config})"
    svrPort="$(${TINYXMLPARSER} -x /Validation/i${refPasswdByIndex}/FTP/Port -f ${config})"
    svrUser="$(${TINYXMLPARSER} -x /Validation/i${refPasswdByIndex}/FTP/User -f ${config})"

    if [ "${svrAddr}" != "${address}" -o "${svrPort}" != "${port}" -o "${svrUser}" != "${username}" ]; then
        resMsg="invalidPasswdRefer"
        return
    fi

    passwd="$(${TINYXMLPARSER} -x /Validation/i${refPasswdByIndex}/FTP/Password -f ${config})"
}

generate_https_necessary_file()
{
	gencert self > /dev/null
	chmod 600 /etc/conf.d/privatekey.pem
	gencert certinfo > /dev/null
}

check_smtp_over_https()
{
	if [ "$sslmode" = "1" ] && [ -f "$SMTPS_CONF" ]; then
		# check smtps.conf connect settings
		smtp_new_address="$address:$port"
		smtp_org_address=`grep "connect" $SMTPS_CONF | sed -n 's/connect = //p'`
		if [ "$smtp_new_address" != "$smtp_org_address" ]; then
			grep -v "connect" $SMTPS_CONF > $TEMP_FILE 
			echo "connect = $smtp_new_address" >> $TEMP_FILE
			mv $TEMP_FILE $SMTPS_CONF
		fi 
				
		if [ -f $pemfile ] && [ -f $certfile ]; then
			`$STUNNEL_PROG $SMTPS_CONF`
			if [ "$?" != "0" ];then
				generate_https_necessary_file
				`$STUNNEL_PROG $SMTPS_CONF`
			fi
		else
			generate_https_necessary_file
			`$STUNNEL_PROG $SMTPS_CONF` 
		fi
	fi
}

generate_test_file()
{
    if [ ! -f "$SEND_FILE" ]; then
        echo "[NOTIFICATION]The Result of Server Test of Your IP Camera" > "$SEND_FILE"
    fi
}

prepare_protocol_over_stunnel()
{
	CONF="$1"
	TARGET_ADDRESS="$2"
	TARGET_PORT="$3"

	tmpFile="$CONF.bak"

	if [ -f "$CONF" ]; then
		new_address="$TARGET_ADDRESS:$TARGET_PORT"
		org_address=`grep "connect" $CONF | sed -n 's/connect = //p'`
		if [ "$new_address" != "$org_address" ]; then
			grep -v "connect" $CONF > $tmpFile
			echo "connect = $new_address" >> $tmpFile
			mv $tmpFile $CONF
		fi

		if [ -f $pemfile ] && [ -f $certfile ]; then
			`$STUNNEL_PROG $CONF`
			if [ "$?" != "0" ];then
				generate_https_necessary_file
				`$STUNNEL_PROG $CONF`
			fi
		else
			generate_https_necessary_file
			`$STUNNEL_PROG $CONF`
		fi
	fi
}

emailSvr()
{
    exitStatus=0

	if [ -n "$address" ] && [ -n "$recipientemail" ]; then
        title="[NOTIFICATION]The Result of Server Test of Your IP Camera"
	    mime_type="-M 2"

	    if [ -n "$username" ]; then
		    auth="-U ${username} -p ${passwd}"
        else
            auth=""
        fi

		if [ "$sslmode" = "1" ]; then
			check_smtp_over_https
			$SMTPC -s "$title" $mime_type -b $SEND_FILE -f "$senderemail" -S "127.0.0.1" -P "25" $auth "$recipientemail" $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
		else
			$SMTPC -s "$title" $mime_type -b $SEND_FILE -f "$senderemail" -S "$address" -P $port $auth "$recipientemail" $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
		fi

		if [ "$?" = "0" ]; then
            resMsg="sendEmailSuccess"
        else
            resMsg="sendEmailFail"
            exitStatus=1
		fi

        if [ "$sslmode" = "1" ]; then
		    kill -9 `cat $pidfile`
	    fi
	else
        resMsg="errEmailAddr"
        exitStatus=1
	fi
}

ftpSvr()
{
    exitStatus=0

    if [ -n "$address" ]; then
	    if [ -n "$username" ]; then
            auth="-u $username -p $passwd"
	    else
            auth=""
        fi
    	
	    if [ ! -n "$port" ]; then
            port="21"
	    fi

		if [ "$passive" = "1" ]; then
			mode=""
		else
			. $NET_STATUS
			mode="-a $IP_ADDR"
		fi

        if [ -z $location ]; then
			DIR="."
		else
			DIR="$location"
		fi

		if [ -x ${FTPS_PROG} ]; then
			${FTPS_PROG} $mode -t 60 $auth -P $port $address "$DIR" $SEND_FILE $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT > /dev/null
		else
			${FTPMPUT_PROG} $mode -t 60 $auth -P $port $address "$DIR" $SEND_FILE $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT > /dev/null
		fi

		if [ "$?" = "0" ]; then
            resMsg="sendFtpSuccess"
	    else
            resMsg="sendFtpFail"
            exitStatus=1
       	fi
    else
        resMsg="errFtpAddr"
        exitStatus=1
    fi
}

httpSvr()
{
    exitStatus=0

    if [ -n "$url" ]; then
        HTTP_URL="$url"
	    case "$HTTP_URL" in
	        http://*)
				HTTP_URL=${HTTP_URL#http://}
				OVER_STUNNEL="false"
				;;
			https://*)
				HTTP_URL=${HTTP_URL#https://}
				OVER_STUNNEL="true"
				;;
		esac

        TMP_IP_PORT=${HTTP_URL%%/*}
		if [ "`echo ${TMP_IP_PORT} | sed -e 's/^\(.\{1\}\).*$/\1/'`" == "[" ]; then
			if [ "`echo ${TMP_IP_PORT} | sed -e 's/\(^.*\)\(.$\)/\2/'`" == "]" ]; then
				HTTP_IP=`echo ${TMP_IP_PORT} | sed 's/^.\(.*\).$/\1/'`
				HTTP_PORT=80
			else
				TMP_IP_PORT=`echo ${TMP_IP_PORT} | sed 's/\(.*\):/\1 /'`
				HTTP_IP=`echo ${TMP_IP_PORT} | cut -d ' ' -f 1 | sed 's/^.\(.*\).$/\1/'`
				HTTP_PORT=`echo ${TMP_IP_PORT} | cut -d ' ' -f 2`
			fi

			IPv6="v6"
		else
			HTTP_IP=${TMP_IP_PORT%%:*}
			if [ "$HTTP_IP" != "$TMP_IP_PORT" ]; then
				HTTP_PORT=${TMP_IP_PORT#$HTTP_IP:}
			else
				HTTP_PORT=80
			fi
		fi

        HTTP_URL="/${HTTP_URL#*/}"

		if [ "${OVER_STUNNEL}" = "true" ] && [ "${cURL}" != "true" ]; then
			prepare_protocol_over_stunnel ${HTTPS_CLIENT_CONF} ${HTTP_IP} ${HTTP_PORT}
			HTTP_IP=127.0.0.1
			HTTP_PORT=26
			IPv6=""
		fi

		HTTP_USERNAME=$username
		HTTP_PASSWD=$passwd

		if [ "${cURL}" = "true" ]; then
			[ "${IPv6}" = "v6" ] && HTTP_IP="[${HTTP_IP}]"
			http_ret_code=$("${CURL_PROG}" -s -o "/dev/null" -w "%{http_code}" "${type}://${HTTP_IP}:${HTTP_PORT}${HTTP_URL}" -u "${HTTP_USERNAME}:${HTTP_PASSWD}" -F "data=@${SEND_FILE}" -k --connect-timeout 25)
		else
			$HTTP_CLIENT_PROG -M UPLOAD -t Text -file ${SEND_FILE} -S ${HTTP_IP} ${HTTP_PORT} "${HTTP_USERNAME}" "${HTTP_PASSWD}" ${IPv6} -U "${HTTP_URL}" > /dev/null
		fi

		if [ "$?" = "0" -a "${cURL}" = "" -o "${cURL}" = "true" -a "${http_ret_code}" = "200" ]; then
            resMsg="sendHttpSuccess"
		else
            resMsg="sendHttpFail"
            exitStatus=1
		fi

        [ -f "${HTTPS_CLIENT_PID_FILE}" ] && kill -9 "$(cat ${HTTPS_CLIENT_PID_FILE})" && rm -f ${HTTPS_CLIENT_PID_FILE}
	else
        resMsg="errHttpAddr"
        exitStatus=1
	fi
}

nasSvr()
{
    exitStatus=0
    smb_path="`echo $location | sed -e 's#[\]#/#g'`"
	tmp_path=${smb_path#//}
	smb_host=${tmp_path%%/*}
	tmp_path=${smb_path#//*/}
	smb_sharefolder=${tmp_path%%/*}
	smb_childfolder=${smb_path#//$smb_host/$smb_sharefolder}
	smb_path1="//${smb_host}/${smb_sharefolder}"	

	destination_path="/mnt/auto/i99"
	if [ -n "$username" ]; then
		if [ -n "$workgroup" ]; then
			parameter="sec=ntlm,username=$username,password=$passwd,workgroup=$workgroup"
		else
			parameter="sec=ntlm,username=$username,password=$passwd"
		fi
	else
		parameter="sec=none"
	fi
	echo "i99 -fstype=cifs,$parameter :$smb_path1" >> /mnt/flash/etc/auto.conf
	cd $destination_path

	if [ "$?" = "0" ]; then
		cp $SEND_FILE $destination_path
		if [ "$?" = "0" ]; then
            resMsg="mountNasSuccess"
		else
            resMsg="nasDenied"
			exitStatus=1
		fi

        sleep 1
		cd /mnt/ramdisk
		$UMOUNT $destination_path
	else
        resMsg="mountNasFail"
        exitStatus=1
	fi

    sed -i '/i99 -fstype=cifs/d' /mnt/flash/etc/auto.conf
}

sdCard()
{
    exitStatus=0
    destination_folder="/mnt/auto/CF/test"

	if [ ! -d "$destination_folder" ]; then
		mkdir $destination_folder
	fi

	if [ -d "$destination_folder" ]; then
		cp $SEND_FILE $destination_folder

		if [ "$?" = "0" ]; then
            resMsg="storeSdCardSuccess"
			rm $SEND_FILE
		fi
    else
        resMsg="storeSdCardFail"
        exitStatus=1
    fi
}

# QoS related settings
. /etc/event_functions
get_qos_eventalarm_settings

getEncParam
decParam

parseParam
if [ "${refTarget}" = "validation" ]; then
    updateValidationPasswd
else
    updateReportPushPasswd
fi

if [ "${resMsg}" != "" ]; then
    exitStatus=1
elif [ "$(checkNetStatus)" == "true" ]; then
    generate_test_file

    case "$type" in
        "email")
            emailSvr
            ;;
        "ftp")
            ftpSvr
            ;;
        "http" | "https")
            httpSvr
            ;;
        "ns")
            nasSvr
            ;;
	    "sdcard")
            sdCard
            ;;
    esac
else
    resMsg="checkNetFail"
    exitStatus=1
fi

response
exit ${exitStatus}
