#!/bin/bash
upgrade_fs_version()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade tracker filesystem: upgrade from ${1} to ${2} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade tracker filesystem: upgrade from ${1} to ${2}
	if [ ${1} -eq 0 ]; then
		if [ ! -d "/mnt/flash2/vcadb" ]; then
			if [ -d "/mnt/flash2/stereotrackerdb" ]; then
				mv -f /mnt/flash2/stereotrackerdb /mnt/flash2/vcadb
			elif [ -d "/mnt/flash2/trackerdb" ]; then
				mv -f /mnt/flash2/trackerdb /mnt/flash2/vcadb
			elif [ -d "/mnt/flash2/countingdb" ]; then
				mv -f /mnt/flash2/countingdb /mnt/flash2/vcadb
			fi
		else
			if [ -d "/mnt/flash2/stereotrackerdb" ]; then
				cp -a /mnt/flash2/stereotrackerdb/*.db /mnt/flash2/vcadb
				rm /mnt/flash2/stereotrackerdb/*
				rmdir /mnt/flash2/stereotrackerdb
			elif [ -d "/mnt/flash2/trackerdb" ]; then
				cp -a /mnt/flash2/trackerdb/*.db  /mnt/flash2/vcadb
				rm /mnt/flash2/trackerdb/*
				rmdir /mnt/flash2/trackerdb
			elif [ -d "/mnt/flash2/countingdb" ]; then
				cp -a /mnt/flash2/countingdb/*.db  /mnt/flash2/vcadb
				rm /mnt/flash2/countingdb/*
				rmdir /mnt/flash2/countingdb
			fi
		fi
		if [ ! -f "/mnt/flash2/vcadb/counting.db" ]; then
			if [ -f "/mnt/flash2/vcadb/vadp.db" ]; then
				mv -f /mnt/flash2/vcadb/vadp.db /mnt/flash2/vcadb/counting.db
			fi
		fi
	elif [ ${1} -eq 1 ]; then
		echo no change
	fi
}

downgrade_fs_version()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade tracker filesystem: downgrade from ${1} to ${2} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade tracker filesystem: downgrade from ${1} to ${2}
	if [ ${1} -eq 1 ]; then
		echo downgrade not allowed >> $LOG
		echo downgrade not allowed 
	elif [ ${1} -eq 0 ]; then
		echo no change
	fi
}

# ${1}:tracker filesystem version
patch_filesystem()
{
	if [ -f $tracker_filesystem_version_file ]; then
		now_fs_version=$(cat $tracker_filesystem_version_file)
	else
		now_fs_version=0
	fi
	if [ $now_fs_version -eq ${1} ]; then
			echo $(date +"%Y-%m-%d %T") -- tracker filesystem version'('$now_fs_version')' is update to date >> $LOG
			echo $(date +"%Y-%m-%d %T") -- tracker filesystem version'('$now_fs_version')' is update to date
	elif [ $now_fs_version -lt ${1} ]; then
		while [ $now_fs_version -lt ${1} ]
		do
			echo $(date +"%Y-%m-%d %T") -- tracker filesystem version'('$now_fs_version')' is behind latest code version'('${1}')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- tracker filesystem version'('$now_fs_version')' is behind latest code version'('${1}')'
			upgrade_fs_version $now_fs_version $((now_fs_version+1))
			now_fs_version=$((now_fs_version+1))
			echo $now_fs_version > $tracker_filesystem_version_file
			echo $(date +"%Y-%m-%d %T") -- after upgrade tracker filesystem version'('$now_fs_version')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- after upgrade tracker filesystem version'('$now_fs_version')'
		done
	else
		while [ $now_fs_version -gt ${1} ]
		do
			echo $(date +"%Y-%m-%d %T") -- tracker filesystem version'('$now_fs_version')' is ahead latest code version'('${1}')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- tracker filesystem version'('$now_fs_version')' is ahead latest code version'('${1}')'
			downgrade_fs_version $now_fs_version $((now_fs_version-1))
			now_fs_version=$((now_fs_version-1))
			echo $now_fs_version > $tracker_filesystem_version_file
			echo $(date +"%Y-%m-%d %T") -- after downgrade tracker filesystem version'('$now_fs_version')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- after downgrade tracker filesystem version'('$now_fs_version')'
		done
	fi
}

# ${1}:online db
repair_online_db()
{
	if [ ! -f ${1} ]
	then
		echo $(date +"%Y-%m-%d %T") -- repair online db ${1} not exist >> $LOG
		echo $(date +"%Y-%m-%d %T") -- repair online db ${1} not exist
	else
		db_version=$(sqlite3 -line ${1} "pragma user_version;" | awk -F'= ' '{print $2}')
		echo $(date +"%Y-%m-%d %T") -- ${1} before repair version'('$db_version')' >> $LOG
		echo $(date +"%Y-%m-%d %T") -- ${1} before repair version'('$db_version')'
		
		echo $(date +"%Y-%m-%d %T") -- ${1} corrupt, repair ${1} >> $LOG
		echo $(date +"%Y-%m-%d %T") -- ${1} corrupt, repair ${1}
		sqlite3 ${1} ".dump" | sqlite3 ${1}.bak
		rm -f ${1}
		mv -f ${1}.bak ${1}
		
		sqlite3 -line ${1} "pragma user_version=$db_version;" >> $LOG
	fi
}

create_rule_id_db()
{
	sqlite3 "$DBDirRaw/ruleid.db" "CREATE TABLE IF NOT EXISTS 'rule_id_tb' ('record_time' INTEGER, 'id' INTEGER PRIMARY KEY AUTOINCREMENT, 'rule_name' VARCHAR);"
}

upgrade_counting_version()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade counting db: upgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade counting db: upgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ] || [ ${2} -eq 3 ]; then
		echo no change
	elif [ ${2} -eq 4 ]; then
		sqlite3 ${1} "attach '${1}' as counting;attach '$DBDirRaw/ruleid.db' as ruleid;insert into ruleid.rule_id_tb ('record_time', 'rule_name') select 0, * from (select distinct rule_name from counting.count_event_tb);detach counting;detach ruleid;"
		sqlite3 ${1} "CREATE TABLE 'count_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'object_id' INTEGER, 'left' INTEGER, 'right' INTEGER);"
		sqlite3 ${1} "attach '${1}' as counting;attach '$DBDirRaw/ruleid.db' as ruleid; insert into count_event_tb_bk('id', 'rule_name', 'event_time', 'object_id', 'left', 'right') select b.id, a.rule_name, a.event_time, a.object_id, a.left, a.right from count_event_tb a, (select max(id) id, rule_name from ruleid.rule_id_tb group by rule_name) b where a.rule_name = b.rule_name;detach counting;detach ruleid;"
		sqlite3 ${1} "drop table count_event_tb;"
		sqlite3 ${1} "alter table count_event_tb_bk  rename to count_event_tb;"
	fi
}

upgrade_zone_version()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade zone db: upgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade zone db: upgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ]; then
		sqlite3 ${1} "CREATE TABLE 'zone_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'avg_wait_count' REAL, 'avg_wait_time' REAL, 'total_count' INTEGER, 'new_enter_count' INTEGER, 'total_leave_time' INTEGER, 'visit_count' INTEGER DEFAULT 0);"
		sqlite3 ${1} "insert into zone_event_tb_bk('rule_name', 'event_time', 'avg_wait_count', 'avg_wait_time', 'total_count', 'new_enter_count', 'total_leave_time') select rule_name, event_time, avg_wait_person, avg_wait_time, total_person, new_enter, total_leave_time from zone_event_tb;"
		sqlite3 ${1} "drop table zone_event_tb;"
		sqlite3 ${1} "alter table zone_event_tb_bk  rename to zone_event_tb;"
	elif [ ${2} -eq 1 ]; then
		sqlite3 ${1} "CREATE TABLE 'zone_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'AvgCount' REAL, 'AvgDuration' REAL, 'TotalCount' INTEGER, 'InwardCount' INTEGER, 'SumOutwardDuration' INTEGER, 'VisitCount' INTEGER DEFAULT 0);"
		sqlite3 ${1} "insert into zone_event_tb_bk('rule_name', 'event_time', 'AvgCount', 'AvgDuration', 'TotalCount', 'InwardCount', 'SumOutwardDuration') select rule_name, event_time, avg_wait_count, avg_wait_time, total_count, new_enter_count, total_leave_time from zone_event_tb;"
		sqlite3 ${1} "drop table zone_event_tb;"
		sqlite3 ${1} "alter table zone_event_tb_bk  rename to zone_event_tb;"
	elif [ ${2} -eq 2 ] || [ ${2} -eq 3 ]; then
		echo no change
	elif [ ${2} -eq 4 ]; then
		sqlite3 ${1} "attach '${1}' as zone;attach '$DBDirRaw/ruleid.db' as ruleid;insert into ruleid.rule_id_tb ('record_time', 'rule_name') select 0, * from (select distinct rule_name from zone.zone_event_tb);detach zone;detach ruleid;"
		sqlite3 ${1} "CREATE TABLE 'zone_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'AvgCount' REAL, 'AvgDuration' REAL, 'TotalCount' INTEGER, 'InwardCount' INTEGER, 'SumOutwardDuration' INTEGER, 'VisitCount' INTEGER DEFAULT 0);"
		sqlite3 ${1} "attach '${1}' as zone;attach '$DBDirRaw/ruleid.db' as ruleid; insert into zone_event_tb_bk('id', 'rule_name', 'event_time', 'AvgCount', 'AvgDuration', 'TotalCount', 'InwardCount', 'SumOutwardDuration', 'VisitCount') select b.id, a.rule_name, event_time, AvgCount, AvgDuration, TotalCount, InwardCount, SumOutwardDuration, VisitCount from zone_event_tb a, (select max(id) id, rule_name from ruleid.rule_id_tb group by rule_name) b where a.rule_name = b.rule_name;detach zone;detach ruleid;"
		sqlite3 ${1} "drop table zone_event_tb;"
		sqlite3 ${1} "alter table zone_event_tb_bk  rename to zone_event_tb;"
	fi
}

upgrade_heatmap_version()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade heatmap db: upgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade heatmap db: upgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ] || [ ${2} -eq 3 ]; then
		echo no change
	fi
}

upgrade_queue_version()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade queue db: upgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade queue db: upgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ]; then
		echo no change
	elif [ ${2} -eq 2 ]; then
		sqlite3 ${1} "CREATE TABLE 'queue_event_tb_bk' ('end_time' INTEGER, 'data' VARCHAR);"
		sqlite3 ${1} "insert into queue_event_tb_bk('end_time', 'data') select end_time, data from queue_event_tb;"
		sqlite3 ${1} "drop table queue_event_tb;"
		sqlite3 ${1} "alter table queue_event_tb_bk  rename to queue_event_tb;"
	elif [ ${2} -eq 3 ]; then
		sqlite3 ${1} "CREATE TABLE 'queue_event_tb_bk' ('rule_name' TEXT, 'start_time' INTEGER, 'end_time' INTEGER, 'data' VARCHAR);"
		sqlite3 ${1} "insert into queue_event_tb_bk('end_time', 'data') select end_time, data from queue_event_tb;"
		sqlite3 ${1} "drop table queue_event_tb;"
		sqlite3 ${1} "alter table queue_event_tb_bk  rename to queue_event_tb;"
	elif [ ${2} -eq 4 ]; then
		sqlite3 ${1} "attach '${1}' as queue;attach '$DBDirRaw/ruleid.db' as ruleid;insert into ruleid.rule_id_tb ('record_time', 'rule_name') select 0, * from (select distinct rule_name from queue.queue_event_tb);detach queue;detach ruleid;"
		sqlite3 ${1} "CREATE TABLE 'queue_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'start_time' INTEGER, 'end_time' INTEGER, 'data' VARCHAR);"
		sqlite3 ${1} "attach '${1}' as queue;attach '$DBDirRaw/ruleid.db' as ruleid; insert into queue_event_tb_bk('id', 'rule_name', 'start_time', 'end_time', 'data') select b.id, a.rule_name, start_time, end_time, data from queue_event_tb a, (select max(id) id, rule_name from ruleid.rule_id_tb group by rule_name) b where a.rule_name = b.rule_name;detach queue;detach ruleid;"
		sqlite3 ${1} "drop table queue_event_tb;"
		sqlite3 ${1} "alter table queue_event_tb_bk  rename to queue_event_tb;"
	fi
}

upgrade_passerby_version()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade passerby db: upgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade passerby db: upgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ] || [ ${2} -eq 3 ]; then
		echo no change
	elif [ ${2} -eq 4 ]; then
		sqlite3 ${1} "attach '${1}' as passerby;attach '$DBDirRaw/ruleid.db' as ruleid;insert into ruleid.rule_id_tb ('record_time', 'rule_name') select 0, * from (select distinct rule_name from passerby.passerby_event_tb);detach passerby;detach ruleid;"
		sqlite3 ${1} "CREATE TABLE 'passerby_event_tb_bk' ('event_time' INTEGER, 'id' INTEGER, 'rule_name' TEXT, 'data' VARCHAR);"
		sqlite3 ${1} "attach '${1}' as passerby;attach '$DBDirRaw/ruleid.db' as ruleid; insert into passerby_event_tb_bk('id', 'rule_name', 'event_time', 'data') select b.id, a.rule_name, event_time, data from passerby_event_tb a, (select max(id) id, rule_name from ruleid.rule_id_tb group by rule_name) b where a.rule_name = b.rule_name;detach passerby;detach ruleid;"
		sqlite3 ${1} "drop table passerby_event_tb;"
		sqlite3 ${1} "alter table passerby_event_tb_bk  rename to passerby_event_tb;"
	fi
}

downgrade_counting_version()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade counting db: downgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade counting db: downgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ] || [ ${2} -eq 3 ] || [ ${2} -eq 4 ]; then
		echo no change
	elif [ ${2} -eq 5 ]; then
		sqlite3 ${1} "CREATE TABLE 'count_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'object_id' INTEGER, 'left' INTEGER, 'right' INTEGER);"
		sqlite3 ${1} "attach '${1}' as counting;attach '$DBDirRaw/ruleid.db' as ruleid; insert into count_event_tb_bk('rule_name', 'id', 'event_time', 'object_id', 'left', 'right') select b.rule_name, a.id, event_time, object_id, left, right from count_event_tb a join ruleid.rule_id_tb b on a.id = b.id;detach counting;detach ruleid;"
		sqlite3 ${1} "drop table count_event_tb;"
		sqlite3 ${1} "alter table count_event_tb_bk  rename to count_event_tb;"
		sqlite3 ${1} "attach '${1}' as counting;attach '$DBDirRaw/ruleid.db' as ruleid;delete from ruleid.rule_id_tb where rule_name in (select distinct rule_name from counting.count_event_tb);detach counting;detach ruleid;"
	fi
}

downgrade_zone_version()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade zone db: downgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade zone db: downgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 2 ]; then
		sqlite3 ${1} "CREATE TABLE 'zone_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'avg_wait_count' REAL, 'avg_wait_time' REAL, 'total_count' INTEGER, 'new_enter_count' INTEGER, 'total_leave_time' INTEGER, 'visit_count' INTEGER DEFAULT 0);"
		sqlite3 ${1} "insert into zone_event_tb_bk('rule_name', 'event_time', 'avg_wait_count', 'avg_wait_time', 'total_count', 'new_enter_count', 'total_leave_time') select rule_name, event_time, AvgCount, AvgDuration, TotalCount, InwardCount, SumOutwardDuration from zone_event_tb;"
		sqlite3 ${1} "drop table zone_event_tb;"
		sqlite3 ${1} "alter table zone_event_tb_bk  rename to zone_event_tb;"
	elif [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 3 ] || [ ${2} -eq 4 ]; then
		echo no change
	elif [ ${2} -eq 5 ]; then
		sqlite3 ${1} "CREATE TABLE 'zone_event_tb_bk' ('id' INTEGER, 'rule_name' TEXT, 'event_time' INTEGER, 'AvgCount' REAL, 'AvgDuration' REAL, 'TotalCount' INTEGER, 'InwardCount' INTEGER, 'SumOutwardDuration' INTEGER, 'VisitCount' INTEGER DEFAULT 0);"
		sqlite3 ${1} "attach '${1}' as zone;attach '$DBDirRaw/ruleid.db' as ruleid; insert into zone_event_tb_bk('rule_name', 'id', 'event_time', 'AvgCount', 'AvgDuration', 'TotalCount', 'InwardCount', 'SumOutwardDuration', 'VisitCount') select b.rule_name, a.id, event_time, AvgCount, AvgDuration, TotalCount, InwardCount, SumOutwardDuration, VisitCount from zone_event_tb a join ruleid.rule_id_tb b on a.id = b.id;detach zone;detach ruleid;"
		sqlite3 ${1} "drop table zone_event_tb;"
		sqlite3 ${1} "alter table zone_event_tb_bk  rename to zone_event_tb;"
		sqlite3 ${1} "attach '${1}' as zone;attach '$DBDirRaw/ruleid.db' as ruleid;delete from ruleid.rule_id_tb where rule_name in (select distinct rule_name from zone.zone_event_tb);detach zone;detach ruleid;"
	fi
}

downgrade_heatmap_version()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade heatmap db: downgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade heatmap db: downgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ] || [ ${2} -eq 3 ] || [ ${2} -eq 4 ]; then
		echo no change
	fi
}

downgrade_queue_version()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade queue db: downgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade queue db: downgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 4 ]; then
		sqlite3 ${1} "CREATE TABLE 'queue_event_tb_bk' ('end_time' INTEGER, 'data' VARCHAR);"
		sqlite3 ${1} "insert into queue_event_tb_bk('end_time', 'data') select end_time, data from queue_event_tb;"
		sqlite3 ${1} "drop table queue_event_tb;"
		sqlite3 ${1} "alter table queue_event_tb_bk  rename to queue_event_tb;"
	elif [ ${2} -eq 3 ]; then
		sqlite3 ${1} "CREATE TABLE 'queue_event_tb_bk' ('start_time' INTEGER, 'end_time' INTEGER, 'data' VARCHAR);"
		sqlite3 ${1} "insert into queue_event_tb_bk('end_time', 'data') select end_time, data from queue_event_tb;"
		sqlite3 ${1} "drop table queue_event_tb;"
		sqlite3 ${1} "alter table queue_event_tb_bk  rename to queue_event_tb;"
	elif [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ]; then
		echo no change
	elif [ ${2} -eq 5 ]; then
		sqlite3 ${1} "CREATE TABLE 'queue_event_tb_bk' ('rule_name' TEXT, 'start_time' INTEGER, 'end_time' INTEGER, 'data' VARCHAR);"
		sqlite3 ${1} "attach '${1}' as queue;attach '$DBDirRaw/ruleid.db' as ruleid; insert into queue_event_tb_bk('rule_name', 'start_time', 'end_time', 'data') select b.rule_name, start_time, end_time, data from queue_event_tb a join ruleid.rule_id_tb b on a.id = b.id;detach queue;detach ruleid;"
		sqlite3 ${1} "drop table queue_event_tb;"
		sqlite3 ${1} "alter table queue_event_tb_bk  rename to queue_event_tb;"
		sqlite3 ${1} "attach '${1}' as queue;attach '$DBDirRaw/ruleid.db' as ruleid;delete from ruleid.rule_id_tb where rule_name in (select distinct rule_name from queue.queue_event_tb);detach queue;detach ruleid;"
	fi
}

downgrade_passerby_version()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade passerby db: downgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade passerby db: downgrade ${1} version from ${2} to ${3}
	if [ ${2} -eq 0 ] || [ ${2} -eq 1 ] || [ ${2} -eq 2 ] || [ ${2} -eq 3 ] || [ ${2} -eq 4 ]; then
		echo no change
	elif [ ${2} -eq 5 ]; then
		sqlite3 ${1} "CREATE TABLE 'passerby_event_tb_bk' ('event_time' INTEGER, 'rule_name' VARCHAR, 'data' VARCHAR);"
		sqlite3 ${1} "attach '${1}' as passerby;attach '$DBDirRaw/ruleid.db' as ruleid; insert into passerby_event_tb_bk('rule_name', 'event_time', 'data') select b.rule_name, event_time, data from passerby_event_tb a join ruleid.rule_id_tb b on a.id = b.id;detach passerby;detach ruleid;"
		sqlite3 ${1} "drop table passerby_event_tb;"
		sqlite3 ${1} "alter table passerby_event_tb_bk  rename to passerby_event_tb;"
		sqlite3 ${1} "attach '${1}' as passerby;attach '$DBDirRaw/ruleid.db' as ruleid;delete from ruleid.rule_id_tb where rule_name in (select distinct rule_name from passerby.passerby_event_tb);detach passerby;detach ruleid;"
	fi
}

version_upgrade()
{
	echo $(date +"%Y-%m-%d %T") -- upgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- upgrade ${1} version from ${2} to ${3}
	filename=${1##*/}

	if [ $filename == "counting.db" ]; then
		upgrade_counting_version ${1} ${2} ${3}
	elif [ $filename == "zone.db" ]; then
		upgrade_zone_version ${1} ${2} ${3}
	elif [ $filename == "heatmap.db" ]; then
		upgrade_heatmap_version ${1} ${2} ${3}
	elif [ $filename == "queue.db" ]; then
		upgrade_queue_version ${1} ${2} ${3}
	elif [ $filename == "passerby.db" ]; then
		upgrade_passerby_version ${1} ${2} ${3}
	fi
	sqlite3 -line ${1} "pragma user_version=${3};" >> $LOG
}

version_downgrade()
{
	echo $(date +"%Y-%m-%d %T") -- downgrade ${1} version from ${2} to ${3} >> $LOG
	echo $(date +"%Y-%m-%d %T") -- downgrade ${1} version from ${2} to ${3}
	filename=${1##*/}
	
	if [ $filename == "counting.db" ]; then
		downgrade_counting_version ${1} ${2} ${3}
	elif [ $filename == "zone.db" ]; then
		downgrade_zone_version ${1} ${2} ${3}
	elif [ $filename == "heatmap.db" ]; then
		downgrade_heatmap_version ${1} ${2} ${3}
	elif [ $filename == "queue.db" ]; then
		downgrade_queue_version ${1} ${2} ${3}
	elif [ $filename == "passerby.db" ]; then
		downgrade_passerby_version ${1} ${2} ${3}
	fi
	sqlite3 -line ${1} "pragma user_version=${3};" >> $LOG
}

# ${1}:online db
check_version()
{
	echo $(date +"%Y-%m-%d %T") -- check ${1} user_version >> $LOG
	echo $(date +"%Y-%m-%d %T") -- check ${1} user_version
	
	db_version=$(sqlite3 -line ${1} "pragma user_version;" | awk -F'= ' '{print $2}')
	echo $(date +"%Y-%m-%d %T") -- ${1} origin version'('$db_version')' >> $LOG
	echo $(date +"%Y-%m-%d %T") -- ${1} origin version'('$db_version')'

	if [ $db_version -eq $latest_db_version ]; then
			echo $(date +"%Y-%m-%d %T") -- ${1} version'('$db_version')' is update to date >> $LOG
			echo $(date +"%Y-%m-%d %T") -- ${1} version'('$db_version')' is update to date
	elif [ $db_version -lt $latest_db_version ]; then
		if [ $db_version -eq 4 ]; then
			create_rule_id_db
		fi
		while [ $db_version -lt $latest_db_version ]
		do
			echo $(date +"%Y-%m-%d %T") -- ${1} version'('$db_version')' is behind latest db version'('$latest_db_version')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- ${1} version'('$db_version')' is behind latest db version'('$latest_db_version')'
			version_upgrade ${1} $db_version $((db_version+1))
			db_version=$(sqlite3 -line ${1} "pragma user_version;" | awk -F'= ' '{print $2}')
			echo $(date +"%Y-%m-%d %T") -- ${1} after upgrade version'('$db_version')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- ${1} after upgrade version'('$db_version')'
		done
		echo "PRAGMA temp_store_directory=$DBDir;vacuum;"
		sqlite3 ${1} "PRAGMA temp_store_directory=$DBDir;vacuum;"
	else
		while [ $db_version -gt $latest_db_version ]
		do
			echo $(date +"%Y-%m-%d %T") -- ${1} version'('$db_version')' is behind latest db version'('$latest_db_version')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- ${1} version'('$db_version')' is behind latest db version'('$latest_db_version')'
			version_downgrade ${1} $db_version $((db_version-1))
			db_version=$(sqlite3 -line ${1} "pragma user_version;" | awk -F'= ' '{print $2}')
			echo $(date +"%Y-%m-%d %T") -- ${1} after downgrade version'('$db_version')' >> $LOG
			echo $(date +"%Y-%m-%d %T") -- ${1} after downgrade version'('$db_version')'
		done
		echo "PRAGMA temp_store_directory=$DBDir;vacuum;"
		sqlite3 ${1} "PRAGMA temp_store_directory=$DBDir;vacuum;"
	fi
}

upgrade_db()
{
	DB=${1}

	if [ ! -f ${1} ]
	then
		echo $(date +"%Y-%m-%d %T") -- ${1} not exist >> $LOG
		echo $(date +"%Y-%m-%d %T") -- ${1} not exist
	else
		check_version ${1} ${2}
		sqlite3 -line ${1} ".exit"
	fi
}


max_log_size=1000000
tracker_filesystem_version_file=/mnt/flash2/tracker_filesystem_version

if [ $# -eq 5 ]; then
	online_db_path=$1
	latest_db_version=$2
	code_fs_version=$3
	cmdtype=$4
	DBDir="\"${5}\""
	DBDirRaw=$5
	LOG="${5}/db_version_ctrl.log"
	mkdir -p $5
	touch $LOG
	log_size=$(wc -c <$LOG)
	echo $(date +"%Y-%m-%d %T") -- lof file size'('$log_size')'
	
	if [ $log_size -gt $max_log_size ]; then
		rm $LOG
	fi
	
	
	if [ "$cmdtype" == "repair" ]; then
		repair_online_db $online_db_path
	elif [ "$cmdtype" == "upgrade" ]; then
		upgrade_db $online_db_path $latest_db_version
	elif [ "$cmdtype" == "patch_filesystem" ]; then
		patch_filesystem $code_fs_version
	fi
	
else
	echo "invalid argument, usage: sh db_version_ctrl.sh (online_db_path) (latest_version) (repair/upgrade)"
fi

exit 0
